# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('feedme', '0007_poll_group'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='answer',
            options={'permissions': (('vote_poll', 'Vote Poll'),)},
        ),
        migrations.AlterModelOptions(
            name='order',
            options={'get_latest_by': 'date', 'permissions': (('view_order', 'View Order'),)},
        ),
        migrations.AlterModelOptions(
            name='orderline',
            options={'verbose_name': 'Orderline', 'verbose_name_plural': 'Orderlines'},
        ),
        migrations.AlterModelOptions(
            name='poll',
            options={'permissions': (('view_poll', 'View Poll'),)},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('feedme', '0002_auto_20141026_1420'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='use_validation',
            field=models.BooleanField(default=True, verbose_name='Enable funds validation'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='orderline',
            name='price',
            field=models.IntegerField(max_length=4, verbose_name='price'),
            preserve_default=True,
        ),
    ]



from django.conf.urls import url
from tastypie.resources import ModelResource
from tastypie import fields
from tastypie.utils import trailing_slash
from feedme.models import Poll, Restaurant, Answer, Order, OrderLine


class RestaurantResource(ModelResource):
    class Meta:
        queryset = Restaurant.objects.all()
        resource_name = 'restaurant'


class VoteResource(ModelResource):
    restaurant = fields.ToOneField(RestaurantResource, 'answer', full=True, null=True)

    class Meta:
        queryset = Answer.objects.all()
        resource_name = 'vote'


class PollResource(ModelResource):
    votes = fields.ToManyField(VoteResource, 'votes', full=True, null=True)

    class Meta:
        queryset = Poll.objects.all()
        resource_name = 'poll'

    def prepend_urls(self):
        """ Adds the following urls to the base resoucre """
        return [
            url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w/-]*)/winner%s$" % (self._meta.resource_name, trailing_slash()), self.wrap_view('winner'), name="api_poll_winner")
        ]

    def winner(self, request, **kwargs):
        """ proxy for the poll winner method """

        self.method_check(request, allowed=['get'])

        # Bundle for the method
        basic_bundle = self.build_bundle(request=request)

        poll = self.cached_obj_get(bundle=basic_bundle, **self.remove_api_resource_names(kwargs))

        return self.create_response(request, poll.get_winner())


class OrderResource(ModelResource):

    class Meta:
        queryset = Order.objects.all()
        resource_name = 'order'


class OrderLineResource(ModelResource):
    order = fields.ForeignKey(OrderResource, 'order')

    class Meta:
        queryset = OrderLine.objects.all()
        resource_name = 'orderline'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('feedme', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('answer', models.ForeignKey(related_name='answer', to='feedme.Restaurant')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Poll',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('question', models.CharField(max_length=250, verbose_name='question')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('due_date', models.DateTimeField(verbose_name='due date')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='answer',
            name='poll',
            field=models.ForeignKey(related_name='poll', to='feedme.Poll'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='answer',
            name='user',
            field=models.ForeignKey(related_name='user', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
    ]



from django.contrib import admin

from feedme.models import Answer, Balance, Order, OrderLine, Poll, Restaurant, Transaction


class DepositWithdrawalFilter(admin.SimpleListFilter):
    """
    A simple filter to select deposits, withdrawals or empty transactions
    """
    title = 'transaction type'

    parameter_name = 'amount'

    def lookups(self, request, model_admin):
        """
        Tuples with values for url and display term
        """
        return (
            ('positive', 'Deposit'),
            ('negative', 'Withdrawal'),
            ('empty', 'Empty')
        )

    def queryset(self, request, queryset):
        if self.value() == 'positive':
            return queryset.filter(amount__gt=0)

        if self.value() == 'negative':
            return queryset.filter(amount__lt=0)

        if self.value() == 'empty':
            return queryset.filter(amount=0)


class OrderLineAdmin(admin.ModelAdmin):
    model = OrderLine
    fieldsets = (
        ('Order Line', {
            'fields': ('order', 'creator', 'users', 'menu_item', 'soda', 'extras', 'price',)
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('paid_for',)
        })
    )
    list_display = (
        'order', 'creator', 'menu_item', 'soda', 'extras',
        'price', 'price_to_pay', 'total_price', 'paid_for', 'num_users'
    )
    list_filter = (
        'paid_for', 'users', 'order',
    )

    def num_users(self, instance):
        return instance.get_num_users()
    num_users.short_description = "Num users"

    def price_to_pay(self, instance):
        return instance.get_price_to_pay()
    price_to_pay.short_description = "Price per user"

    def total_price(self, instance):
        return instance.get_total_price()
    total_price.short_description = "Total price"


class OrderLineInline(admin.TabularInline):
    model = OrderLine
    fields = (
        'creator', 'users', 'menu_item', 'soda', 'extras', 'price', 'paid_for',
    )
    extra = 0


class OrderAdmin(admin.ModelAdmin):
    model = Order
    fieldsets = (
        ('Order', {
            'fields': ('group', 'restaurant', 'date', 'extra_costs')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': ('active', 'use_validation')
        })
    )
    list_display = (
        'restaurant', 'date', 'group', 'extra_costs', 'active', 'use_validation',
        'total_cost',
    )
    list_filter = (
        'restaurant', 'date', 'group', 'active', 'use_validation',
    )
    search_fields = (
        'restaurant', 'date', 'group',
    )
    inlines = [
        OrderLineInline
    ]

    def total_cost(self, instance):
        return instance.get_total_sum
    total_cost.short_description = "Total cost"


class AnswerInline(admin.TabularInline):
    model = Answer
    extra = 0


class PollAdmin(admin.ModelAdmin):
    model = Poll
    inlines = [
        AnswerInline
    ]


class TransactionAdmin(admin.ModelAdmin):
    model = Transaction
    fieldsets = (
        ('Transaction', {
            'fields': ('user', 'amount')
        }),
        ('Advanced', {
            'classes': ('collapse',),
            'fields': ('date',)
        })
    )
    list_display = (
        'user', 'date', 'amount',
    )
    list_filter = (
        'user', 'date', DepositWithdrawalFilter,
    )
    search_fields = (
        'user', 'date', 'amount',
    )

    def get_queryset(self, request):  # Only allow to see own if not superuser
        qs = super(TransactionAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(user=request.user)


class BalanceAdmin(admin.ModelAdmin):
    model = Balance
    readonly_fields = ('user',)
    list_display = (
        'user', 'balance'
    )

    def balance(self, instance):
        return instance.get_balance()

    def get_queryset(self, request):  # Only allow to see own if not superuser
        qs = super(BalanceAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(user=request.user)

admin.site.register(Balance, BalanceAdmin)
admin.site.register(Order, OrderAdmin)
admin.site.register(OrderLine, OrderLineAdmin)
admin.site.register(Poll, PollAdmin)
admin.site.register(Restaurant)
admin.site.register(Transaction, TransactionAdmin)



# -*- coding: utf-8 -*-

from django.contrib import messages
from django.contrib.auth.models import Group
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, get_object_or_404, redirect
from django.db.models import Q
from django.views.generic import DetailView

import logging

from feedme.models import OrderLine, Order, Restaurant, Balance, Poll, Answer
from feedme.forms import (
    OrderLineForm, OrderForm, ManageOrderForm, NewOrderForm,
    NewRestaurantForm, ManageBalanceForm, NewPollForm, PollAnswerForm)
from feedme.utils import (
    get_feedme_groups, get_next_tuesday, get_next_wednesday, get_order, get_poll, is_admin,
    in_other_orderline, manually_parse_users, validate_user_funds, check_orderline, handle_payment
)

logger = logging.getLogger(__name__)
try:
    # Django 1.7 way for importing custom user
    from django.contrib.auth import AUTH_USER_MODEL
    User = get_user_model()
except ImportError:
    try:
        # Django 1.6 way for importing custom user, will crash in Django 1.7
        from django.contrib.auth import get_user_model
        User = get_user_model()
    except AppRegistryNotReady:
        # If all else fails, import default user model -- please report this bug
        from django.contrib.auth.models import User


# Index
def index(request):
    r = dict(
        is_admin=is_admin(request),
        feedme_groups=[g for g in get_feedme_groups() if request.user in g.user_set.all()],
    )
    return render(request, 'feedme/index.html', r)


@login_required()
def index_new(request, group=None):
    group = get_object_or_404(Group, name=group) if \
        request.user in get_object_or_404(Group, name=group).user_set.all() else None
    order = get_order(group) if group else None
    poll = get_poll(group) if group else None
    r = dict()
    r['group'] = group
    r['order'] = order
    r['restaurants'] = Restaurant.objects.all()
    r['is_admin'] = is_admin(request)
    r['can_join'] = not in_other_orderline(order, request.user)
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]

    a_id = None
    if str(request.user) != 'AnonymousUser':
        if Answer.objects.filter(poll=poll, user=request.user).count() == 1:
            a_id = Answer.objects.get(poll=poll, user=request.user)

    if request.method == 'POST':
        if request.POST['act'] == 'vote':
            if a_id is not None:
                form = PollAnswerForm(request.POST, instance=a_id)
            else:
                form = PollAnswerForm(request.POST)
            if form.is_valid():
                answer = form.save(commit=False)
                answer.user = request.user
                answer.poll = poll
                answer.save()
                messages.success(request, 'Voted for %s' % answer.answer)
                return redirect('feedme:feedme_index_new', group)

    if poll is not None:
        r['poll'] = poll
        if a_id is None:
            r['answer'] = PollAnswerForm()
        else:
            r['answer'] = PollAnswerForm(instance=a_id)
        r['results'] = poll.get_result()

    return render(request, 'feedme/index.html', r)


# View order
@login_required()
def orderview(request, order_id=None, group=None):
    if order_id:
        order = get_object_or_404(Order, pk=order_id)
    else:
        order = Order()

    group = get_object_or_404(Group, name=group)

    if request.method == 'POST':
        form = OrderForm(request.POST, instance=order)
        if form.is_valid():
            form = form.save(commit=False)
            form.creator = request.user
            form.order_line = get_order(group)
            form.save()
            messages.success(request, 'Orderline added')
            return redirect('feedme:feedme_index_new', group)

        form = OrderForm(request.POST)
    else:
        form = OrderForm(instance=order)

    r = dict()
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['form'] = form
    r['is_admin'] = is_admin(request)

    return render(request, 'feedme/orderview.html', r)


# New / edit order line
def orderlineview(request, orderline_id=None, group=None):
    new_or_existing_orderline = 'new'
    if orderline_id is None:
        orderline = OrderLine()
        creator = None
    else:
        orderline = get_object_or_404(OrderLine, pk=orderline_id)
        new_or_existing_orderline = 'existing'
        creator = orderline.creator

    group = get_object_or_404(Group, name=group)

    if request.method == 'POST':
        form = OrderLineForm(request.POST, instance=orderline)
        if form.is_valid():
            new_orderline = form.save(commit=False)
            if creator is not None:
                new_orderline.creator = creator
            else:
                new_orderline.creator = request.user
            new_orderline.order = get_order(group)
            users = manually_parse_users(form)
            if check_orderline(group, creator, orderline.price, buddies=users):
                new_orderline.save()
                form.save_m2m()  # Manually save the m2m relations when using commit=False
                if new_or_existing_orderline == 'new':
                    messages.success(request, "Orderline added")
                else:
                    messages.success(request, "Orderline edited")
                new_orderline.users.add(new_orderline.creator)
                return redirect('feedme:feedme_index_new', group)
            else:
                messages.error(request, "Orderline validation failed, please verify your data and try again.")
    else:
        form = OrderLineForm(instance=orderline)
        form.fields["users"].queryset = get_order(group).available_users().exclude(id=request.user.id)

    r = dict()
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['form'] = form
    r['is_admin'] = is_admin(request)

    return render(request, 'feedme/orderview.html', r)


class OrderlineDetail(DetailView):
    model = OrderLine

    template_name = 'feedme/orderview.html'

    def get_queryset(self):
        print(self.args)
        self.orderline = get_object_or_404(OrderLine, pk=self.args[0])
        return self.orderline


def create_orderline(request, group=None):
    r = dict()
    group = get_object_or_404(Group, name=group)

    if request.method == 'POST':
        form = OrderLineForm(request.POST)
        if form.is_valid():
            new_orderline = form.save(commit=False)
            new_orderline.creator = request.user
            new_orderline.order = get_order(group)
            users = manually_parse_users(form)
            if check_orderline(group, new_orderline.creator, new_orderline.price, buddies=users):
                new_orderline.save()
                form.save_m2m()  # Manually save the m2m relations when using commit=False
                new_orderline.users.add(new_orderline.creator)
                messages.success(request, 'Added orderline')
                return redirect('feedme:feedme_index_new', group)
            else:
                messages.error(request, 'Failed to add orderline')
    else:
        form = OrderLineForm()
        form.fields["users"].queryset = get_order(group).available_users().exclude(id=request.user.id)

    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['form'] = form
    r['is_admin'] = is_admin(request)

    return render(request, 'feedme/orderview.html', r)


# Edit order line
def edit_orderline(request, group, orderline_id):
    orderline = get_object_or_404(OrderLine, pk=orderline_id)
    group = get_object_or_404(Group, name=group)
    if request.user != orderline.creator and request.user not in orderline.users.all():
        messages.error(request, 'You need to be the creator')
        return redirect('feedme:feedme_index_new', group)
    return orderlineview(request, orderline_id=orderline_id, group=group)


# Delete order line
def delete_orderline(request, group, orderline_id):
    orderline = get_object_or_404(OrderLine, pk=orderline_id)
    group = get_object_or_404(Group, name=group)
    if orderline.creator == request.user:
        orderline.delete()
        messages.success(request, 'Orderline deleted')
    else:
        messages.error(request, 'You need to be the creator or the buddy')
    return redirect('feedme:feedme_index_new', group)


@login_required()
def join_orderline(request, group, orderline_id):
    orderline = get_object_or_404(OrderLine, pk=orderline_id)
    group = get_object_or_404(Group, name=group)
    # @TODO if not buddy system enabled, disable join
    if in_other_orderline(get_order(group), request.user):
        messages.error(request, 'You cannot be in multiple orderlines')
        logger.warn('%s tried to join multiple orderlines (new: %i) for "%s"!' %
                     (request.user, orderline.id, get_order(group)))
    elif orderline.order.use_validation and \
            not validate_user_funds(
                request.user, (orderline.price / (orderline.users.count() + 1))):  # Adds us to the test as well
        messages.error(request, 'You need cashes')
        logger.info('%s tried to join an orderline for "%s", by %s,  with insufficient funds.' %
                    (request.user, orderline.order, orderline.creator))
    else:
        orderline.users.add(request.user)
        orderline.save()
        messages.success(request, 'Joined orderline')
    return redirect('feedme:feedme_index_new', group)


@login_required()
def leave_orderline(request, group, orderline_id):
    orderline = get_object_or_404(OrderLine, pk=orderline_id)
    group = get_object_or_404(Group, name=group)
    if request.user not in orderline.users.all():
        messages.error(request, 'You cannot leave since you are not in the users')
        logger.warn('%s tried to leave orderline#%i for "%s" without being in it!' %
                     (request.user, orderline.id, get_order(group)))
    else:
        orderline.users.remove(request.user)
        orderline.save()
        messages.success(request, 'Left orderline')
    return redirect('feedme:feedme_index_new', group)


@login_required
def order_history(request):
    user_orders = OrderLine.objects.filter(Q(creator=request.user) | Q(users=request.user))
    return render(request, 'feedme/order_history.html', {'order_history': user_orders})

# ADMIN


# New order
@permission_required('feedme.add_order', raise_exception=True)
def new_order(request, group=None):
    logger.warn('DEPRECATED - STOP USING THIS')
    group = get_object_or_404(Group, name=group)
    if request.method == 'POST':
        form = NewOrderForm(request.POST)
        if form.is_valid():
            form.save()
            poll = Poll.get_active()
            if poll:
                poll.deactivate()
            messages.success(request, 'New order added')
            logger.info('%s created a new order for %s: "%s".' % (request.user, group, get_order(group)))
            return redirect('feedme:feedme_index_new', group)
    else:
        form = NewOrderForm()
        if Poll.objects.count() > 0:
            poll = Poll.get_active()
            if poll:
                form.fields['restaurant'].initial = poll.get_winner()
        form.fields["date"].initial = get_next_wednesday()
        form.fields['group'].initial = group
    r = dict()
    r['group'] = group
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['form'] = form
    r['is_admin'] = is_admin(request)
    return render(request, 'feedme/admin.html', r)


def admin(request, group=None):
    # Admin index - defaults to new order page
    r = dict()
    group = get_object_or_404(Group, name=group)

    if request.method == 'POST':
        form = NewOrderForm(request.POST)
        if form.is_valid():
            form.save()
            poll = Poll.get_active()
            if poll:
                poll.deactivate()
            messages.success(request, 'New order added')
            logger.info('%s created a new order for %s: "%s".' % (request.user, group, get_order(group)))
            return redirect('feedme:feedme_index_new', group)
    else:
        form = NewOrderForm()
        if Poll.objects.count() > 0:
            poll = Poll.get_active()
            if poll:
                form.fields['restaurant'].initial = poll.get_winner()
        form.fields["date"].initial = get_next_wednesday()

    # r['form'] = form
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['is_admin'] = is_admin(request)
    return render(request, 'feedme/admin.html', r)


# Manage users (deposit, withdraw, overview)
class ManageUserViewSet(DetailView):
    model = Balance
    fields = ('user', 'get_balance')
    template_name = 'feedme/manage_users.html'

    def get(self, request, group=None):
        r = {}
        group = get_object_or_404(Group, name=group)
        r['group'] = group
        r['form'] = ManageBalanceForm()
        r['form'].fields["user"].queryset = Balance.objects.filter(user__groups=group)

        return render(request, 'feedme/manage_users.html', r)

    def post(self, request, group=None):
        r = {}
        group = get_object_or_404(Group, name=group)
        form = ManageBalanceForm(request.POST)
        if form.is_valid():
            balance = form.cleaned_data['user']
            amount = form.cleaned_data['amount']
            user_balance = [balance.get_balance(), ]
            balance.deposit(amount) if amount > 0 else balance.withdraw(amount * -1)
            user_balance.append(balance.get_balance())
            messages.success(request, 'Successfully deposited %s for %s (Changed from %s to %s)' %
                             (balance.user, amount, user_balance[0], user_balance[1]))
            logger.info('%s deposited %.2f kr for %s. Balance is now %.2f.' %
                        (request.user, amount, balance.user, user_balance[1]))
            return redirect('feedme:manage_users', group)
        else:
            r['form'] = form
            messages.error(request, 'Invalid values supplied, check form error for details.')

        return render(request, 'feedme/manage_users.html', r)


# Manage order (payment handling)
@permission_required('feedme.change_balance', raise_exception=True)
def manage_order(request, group=None):
    r = dict()
    group = get_object_or_404(Group, name=group)
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group

    if request.method == 'POST':
        form = ManageOrderForm(request.POST)
        if form.is_valid():
            data = form.cleaned_data
            order = get_object_or_404(Order, pk=data['orders'].id)
            if 'active_order_submit' in request.POST:
                order.active = request.POST['active_order_submit'] == 'Activate'
                order.save()
                return redirect('feedme:manage_order', group)
            orderlines = order.orderline_set.all()
            total_price = order.extra_costs
            for orderline in orderlines:
                orderline.users.add(orderline.creator)
                orderline.each = orderline.get_price_to_pay()
                total_price += orderline.price
            r['form'] = form
            r['order'] = order
            r['orderlines'] = orderlines
            r['total_price'] = total_price
            if request.POST['act'] == 'Load':
                return render(request, 'feedme/manage_order.html', r)
            elif request.POST['act'] == 'Edit':
                for rq in request.POST:
                    if 'edit_orderline_price' in rq:
                        i = rq.split('-')[1]
                        ol = orderlines.get(id=i)
                        change = int(request.POST[rq])
                        if ol.price != change:
                            ol.price = change
                            ol.save()
                            messages.success(
                                request, 'Changed price for %(ol)s to %(price)s' % {'ol': ol, 'price': ol.price})
                            logger.info('%s changed price of orderline#%i for %s from %.2f to %.2f.' %
                                        (request.user, orderline.id, orderline.creator, (ol.price - change), ol.price))
                return redirect('feedme:manage_order', group=group)
            elif request.POST['act'] == 'Pay':
                paid, existing, negatives = handle_payment(request, order)
                if len(paid) > 0:
                    messages.success(request, 'Paid orderlines for %s.' % ', '.join(paid))
                if len(existing) > 0:
                    messages.error(request, 'Already paid orderlines for %s.' % ', '.join(existing))
                if len(negatives) > 0:
                    messages.error(request, 'These users now have negative balances: %s' % ', '.join(negatives))
                return redirect('feedme:manage_order', group=group)
        else:
            form = ManageOrderForm(request.POST)
    else:
        form = ManageOrderForm()

    active_orders = Order.objects.filter(active=True)
    inactive_orders = Order.objects.exclude(active=True)
    orders = active_orders | inactive_orders
    orders = orders.order_by('-active', '-date')

    form.fields["orders"].queryset = orders

    r['form'] = form
    r['is_admin'] = is_admin(request)
    r['orders'] = orders

    return render(request, 'feedme/manage_order.html', r)


# New restaurant
@permission_required('feedme.add_restaurant', raise_exception=True)
def new_restaurant(request, restaurant_id=None, group=None):
    if restaurant_id is None:
        restaurant = Restaurant()
    else:
        restaurant = get_object_or_404(Restaurant, pk=restaurant_id)

    if request.method == 'POST':
        form = NewRestaurantForm(request.POST, instance=restaurant)
        if form.is_valid():
            form.save()
            messages.success(request, "Restaurant added")
            return redirect('feedme:new_order', group)
        else:
            form = NewRestaurantForm(request.POST)
    else:
        form = NewRestaurantForm(instance=restaurant)

    r = dict()
    group = get_object_or_404(Group, name=group)
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['form'] = form
    r['is_admin'] = is_admin(request)

    return render(request, 'feedme/admin.html', r)


# Edit restaurant
@permission_required('feedme.change_restaurant', raise_exception=True)
def edit_restaurant(request, restaurant_id=None):
    restaurant = get_object_or_404(Restaurant, pk=restaurant_id)
    return new_restaurant(request, restaurant)


@permission_required('feedme.add_poll', raise_exception=True)
def new_poll(request, group=None):
    group = get_object_or_404(Group, name=group)
    if request.method == 'POST':
        form = NewPollForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, 'New poll added')
            logger.info('%s created a new poll for %s.' % (request.user, group))
            return redirect('feedme:feedme_index_new', group)
        else:
            messages.error(request, 'Form not validated')
    else:
        form = NewPollForm()
        form.fields['question'].initial = "Hvor skal %s spise?" % group.name
        form.fields['due_date'].initial = get_next_tuesday()
        form.fields['group'].initial = group

    r = dict()
    r['feedme_groups'] = [g for g in get_feedme_groups() if request.user in g.user_set.all()]
    r['group'] = group
    r['form'] = form
    r['is_admin'] = is_admin(request)

    return render(request, 'feedme/admin.html', r)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('feedme', '0008_auto_20150428_1659'),
    ]

    operations = [
        migrations.AlterField(
            model_name='orderline',
            name='price',
            field=models.IntegerField(verbose_name='price'),
        ),
        migrations.AlterField(
            model_name='orderline',
            name='users',
            field=models.ManyToManyField(related_name='buddies', blank=True, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='restaurant',
            name='email',
            field=models.EmailField(max_length=254, verbose_name='email address', blank=True, null=True),
        ),
    ]



import os
from setuptools import setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='feedme',
    version='1.9.0',
    author='dotkom',
    author_email='dotkom@online.ntnu.no',
    packages=['feedme'],
    license='BSD License',  # example license
    description='Food ordering management system',
    long_description='A food ordering management system for Django.\n\n\
    This project was initially started to help \'dotKom\' with organizing the ordering of food for their work nights.\n\n\
    Check out the github repo for installation instructions.',
    url='https://github.com/dotKom/feedme',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    include_package_data=True,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        ('feedme', '0005_auto_20150218_2016'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='group',
            field=models.ForeignKey(default=4, to='auth.Group'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('feedme', '0009_auto_20150926_2229'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='user',
            field=models.ForeignKey(related_name='bruker', to=settings.AUTH_USER_MODEL),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='restaurant',
            name='email',
            field=models.EmailField(max_length=75, null=True, verbose_name='email address', blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType

from datetime import date, timedelta
import logging

from feedme.models import Balance, Order, OrderLine, Poll

logger = logging.getLogger(__name__)
try:
    # Django 1.7 way for importing custom user
    from django.contrib.auth import AUTH_USER_MODEL
    User = AUTH_USER_MODEL
except ImportError:
    # Django 1.6 way for importing custom user, will crash in Django 1.7
    from django.contrib.auth import get_user_model
    User = get_user_model()


def get_feedme_groups():
    order_ctype = ContentType.objects.get_for_model(Order)
    permission = Permission.objects.get(codename='view_order', content_type=order_ctype)
    return [g for g in Group.objects.all() if permission in g.permissions.all()]


# Get or create balance for a user
def get_or_create_balance(user):
    return Balance.objects.get_or_create(user=user)[0]


# Check that the user has enough funds
def validate_user_funds(user, amount):
    get_or_create_balance(user)
    return user.balance.get_balance() >= amount


# The actual function for payment
def pay(user, amount):
    balance = get_or_create_balance(user)
    balance.withdraw(amount)  # This returns True/False whether or not the payment was possible.
    balance.save()


# Deposit of funds
def handle_deposit(data):
    balance = get_or_create_balance(data['user'])
    print('updating %s' % balance)
    amount = data['amount']
    if amount >= 0:
        balance.deposit(amount)
    else:
        balance.withdraw(amount)


# yes
def get_next_tuesday():
    today = date.today()
    day = today.weekday()
    if day < 1:
        diff = timedelta(days=(1 - day))
    elif day > 1:
        diff = timedelta(days=(7 - day + 1))
    else:
        diff = timedelta(days=0)

    return today + diff


def get_next_wednesday():
    today = date.today()
    day = today.weekday()
    if day < 2:
        diff = timedelta(days=(2 - day))
    elif day > 2:
        diff = timedelta(days=(7 - day + 2))
    else:
        diff = timedelta(days=0)

    return today + diff


def is_admin(request):
    return request.user.has_perm('feedme.change_order')


# Gets latest active order
def get_order(group=None):
    if Order.objects.filter(group=group):
        orders = Order.objects.filter(group=group).order_by('-id')
        for order in orders:
            if order.active:
                return order
    else:
        return None


# Gets latest active poll
def get_poll(group=None):
    if Poll.objects.filter():
        if Poll.objects.filter(group=group, active=True).count() >= 1:
            return Poll.objects.filter(group=group, active=True).order_by('-id')[0]
        else:
            return None


# Manually parses users to validate user funds on buddy-add on initial orderline creation
def manually_parse_users(form):
    li = str(form).split('<select')
    potential_users = li[1].split('<option')
    user_ids = []
    for user in potential_users:
        if 'selected' in user:
            user_ids.append(user.split('value="')[1].split('"')[0])
    users = []
    for i in user_ids:
        users.append(User.objects.get(pk=i))
    return users


# Checks if user is in another orderline
def in_other_orderline(order, user):
    r1 = ""
    r2 = ""
    if order:
        if order.orderline_set:
            if order.orderline_set.filter(creator=user.id):
                r1 = user == order.orderline_set.filter(creator=user.id)[0].creator
            if order.orderline_set.filter(users=user.id):
                r2 = user in order.orderline_set.filter(users=user.id)[0].users.all()
    return r1 or r2


def get_orderline_for_order_and_creator(order, creator):
    created = True
    try:
        orderline = OrderLine.objects.get(order=order, creator=creator)
        created = False
    except OrderLine.DoesNotExist:
        orderline = OrderLine()
        orderline.creator = creator
    return orderline, created


def validate_users_funds(users, price):
    # Temporarily calculate price to pay per user, check if all users have sufficient funds
    price /= len(users)
    unsufficient_funds_users = []
    for user in users:
        if not validate_user_funds(user, price):
            unsufficient_funds_users.append(user)
    return unsufficient_funds_users


# Validation of orderline
def check_orderline(group, creator, price, buddies=None):
    order = get_order(group)

    orderline_exists = False

    orderline, created = get_orderline_for_order_and_creator(order, creator)

    logger.debug('Validating %s orderline for "%s" (%.2f kr) by "%s".' %
                 ("new" if created else "existing", order, price, creator))

    # Update users
    users = [orderline.creator]
    users.extend(buddies)

    unsufficient_funds_users = validate_users_funds(users, price)

    if order.use_validation and unsufficient_funds_users:
        logger.debug("Validation failed. Users unable to join orderline: %s" % unsufficient_funds_users)
        return False
    else:
        logger.debug("Validation successful.")
        return True


# Handle payment
def handle_payment(request, order):
    orderlines = order.orderline_set.all()

    # Initialize empty lists for users
    paid = []
    already_paid = []
    negatives = []

    # Go through each orderline, paying it for each user
    for orderline in orderlines:
        if not orderline.paid_for:
            amount = orderline.get_price_to_pay()
            logger.info('Paying orderline for "%s" (%.2f kr, %.2f kr ea) for "%s".'
                        % (orderline.order, orderline.price, amount, orderline.users.all()))
            # Pay for each user
            for user in orderline.users.all():
                pay(user, amount)
                paid.append(user.get_username())
                if user.balance.get_balance() < 0:
                    negatives.append(user.get_username())
            orderline.paid_for = True
            orderline.save()

        else:
            # Tell that these have already been paid
            already_paid.append(orderline.creator.get_username())
            if orderline.users.all().count() > 0:
                for user in orderline.users.all():
                    if user == orderline.creator:
                        logger.warn('Creator (%s) both in users and creator for orderline #%i' % (user, orderline.id))
                    else:
                        already_paid.append(user.get_username())

    # Disable the order
    order.active = False
    order.save()

    return paid, already_paid, negatives
